@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 The MIT License (MIT)
 
@@ -34,14 +32,16 @@ with extra doc-strings and performance improvements.
 from __future__ import annotations
 
 import base64
+import dataclasses
 import hmac
 import struct
 from hashlib import sha1
 from time import time
-from typing import TYPE_CHECKING, Any, Optional
+from typing import TYPE_CHECKING, Any
 
 from .errors import ConfirmationError
-from .models import community_route
+from .models import URL
+from .utils import Intable
 
 if TYPE_CHECKING:
     from .state import ConnectionState
@@ -55,20 +55,15 @@ __all__ = (
 )
 
 
-def generate_one_time_code(shared_secret: str, timestamp: Optional[int] = None) -> str:
-    """Generate a Steam Guard code for signing in.
+def generate_one_time_code(shared_secret: str, timestamp: int | None = None) -> str:
+    """Generate a Steam Guard code for signing in or at a specific time.
 
     Parameters
     -----------
-    shared_secret: :class:`str`
-        Identity secret from steam guard.
-    timestamp: Optional[:class:`int`]
+    shared_secret
+        Shared secret from steam guard.
+    timestamp
         The unix timestamp to generate the key for.
-
-    Returns
-    --------
-    :class:`str`
-        The desired 2FA code for the timestamp.
     """
     timestamp = timestamp or int(time())
     time_buffer = struct.pack(">Q", timestamp // 30)  # pack as Big endian, uint64
@@ -83,43 +78,34 @@ def generate_one_time_code(shared_secret: str, timestamp: Optional[int] = None)
     return "".join(code)
 
 
-def generate_confirmation_code(identity_secret: str, tag: str, timestamp: Optional[int] = None) -> str:
+def generate_confirmation_code(identity_secret: str, tag: str, timestamp: int | None = None) -> str:
     """Generate a trade confirmation code.
 
     Parameters
     -----------
-    identity_secret: :class:`str`
+    identity_secret
         Identity secret from steam guard.
-    tag: :class:`str`
+    tag
         Tag to encode to.
-    timestamp: Optional[:class:`int`]
+    timestamp
         The time to generate the key for.
-
-    Returns
-    --------
-    :class:`str`
-        Confirmation code for the set timestamp.
     """
     timestamp = timestamp or int(time())
     buffer = struct.pack(">Q", timestamp) + tag.encode("ascii")
     return base64.b64encode(hmac.new(base64.b64decode(identity_secret), buffer, digestmod=sha1).digest()).decode()
 
 
-def generate_device_id(user_id64: str) -> str:
-    """
+def generate_device_id(user_id64: Intable) -> str:
+    """Generate the device id for a user's 64 bit ID.
+
     Parameters
     -----------
-    user_id64: :class:`str`
+    user_id64
         The 64 bit steam id to generate the device id for.
-
-    Returns
-    --------
-    :class:`str`
-        The device id.
     """
     # it works, however it's different that one generated from mobile app
 
-    hexed_steam_id = sha1(user_id64.encode("ascii")).hexdigest()
+    hexed_steam_id = sha1(str(int(user_id64)).encode("ascii")).hexdigest()
     partial_id = [
         hexed_steam_id[:8],
         hexed_steam_id[8:12],
@@ -130,55 +116,66 @@ def generate_device_id(user_id64: str) -> str:
     return f'android:{"-".join(partial_id)}'
 
 
+@dataclasses.dataclass
 class Confirmation:
-    def __init__(self, state: ConnectionState, id: str, data_confid: int, data_key: str, trade_id: int, tag: str):
-        self._state = state
-        self.id = id.split("conf")[1]
-        self.data_confid = data_confid
-        self.data_key = data_key
-        self.tag = tag
-        self.trade_id = trade_id  # this isn't really always the trade ID, but for our purposes this is fine
+    __slots__ = (
+        "_state",
+        "id",
+        "data_conf_id",
+        "data_key",
+        "trade_id",
+    )
+
+    _state: ConnectionState
+    id: str
+    data_conf_id: int
+    data_key: str
+    trade_id: int  # this isn't really always the trade ID, but for our purposes this is fine
 
     def __repr__(self) -> str:
         return f"<Confirmation id={self.id!r} trade_id={self.trade_id}>"
 
-    def __eq__(self, other: Any) -> bool:
+    def __eq__(self, other: object) -> bool:
         return isinstance(other, Confirmation) and self.trade_id == other.trade_id and self.id == other.id
 
-    def _confirm_params(self, tag: str) -> dict:
+    @property
+    def tag(self) -> str:
+        return f"details{self.data_conf_id}"
+
+    def _confirm_params(self, tag: str) -> dict[str, str | int]:
         timestamp = int(time())
         return {
             "p": self._state._device_id,
-            "a": self._state._id64,
+            "a": self._state.client.user.id64,
             "k": self._state._generate_confirmation(tag, timestamp),
             "t": timestamp,
             "m": "android",
             "tag": tag,
         }
 
-    def _assert_valid(self, resp: dict) -> None:
+    def _assert_valid(self, resp: dict[str, Any]) -> None:
         if not resp.get("success", False):
-            self._state._confirmations_to_ignore.append(self.id)
+            self._state._confirmations_to_ignore.append(self.trade_id)
             raise ConfirmationError
 
     async def confirm(self) -> None:
         params = self._confirm_params("allow")
         params["op"] = "allow"
-        params["cid"] = self.data_confid
+        params["cid"] = self.data_conf_id
         params["ck"] = self.data_key
-        resp = await self._state.request("GET", community_route("mobileconf/ajaxop"), params=params)
+        resp = await self._state.http.get(URL.COMMUNITY / "mobileconf/ajaxop", params=params)
         self._assert_valid(resp)
 
     async def cancel(self) -> None:
         params = self._confirm_params("cancel")
         params["op"] = "cancel"
-        params["cid"] = self.data_confid
+        params["cid"] = self.data_conf_id
         params["ck"] = self.data_key
-        resp = await self._state.request("GET", community_route("mobileconf/ajaxop"), params=params)
+        resp = await self._state.http.get(URL.COMMUNITY / "mobileconf/ajaxop", params=params)
         self._assert_valid(resp)
 
-    async def details(self) -> dict:
+    async def details(self) -> str:
         params = self._confirm_params(self.tag)
-        resp = await self._state.request("GET", community_route(f"mobileconf/details/{self.id}"), params=params)
+        resp = await self._state.http.get(URL.COMMUNITY / f"mobileconf/details/{self.id}", params=params)
         self._assert_valid(resp)
         return resp["html"]
