@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 The MIT License (MIT)
 
@@ -26,12 +24,12 @@ SOFTWARE.
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Optional, Union
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
     from .abc import SteamID
     from .clan import Clan
-    from .enums import EFriendRelationship
+    from .enums import FriendRelationship
     from .state import ConnectionState
     from .user import User
 
@@ -44,21 +42,9 @@ __all__ = (
 
 
 class Invite:
-    """Represents a invite from a user.
-
-    Attributes
-    ----------
-    invitee: Union[:class:`~steam.User`, :class:`~steam.SteamID`]
-        The user who sent the invite.
-    relationship: Optional[:class:`~steam.EFriendRelationship`]
-        The relationship you have with the invitee.
-    """
-
     __slots__ = ("invitee", "relationship", "_state")
 
-    def __init__(
-        self, state: ConnectionState, invitee: Union[User, SteamID], relationship: Optional[EFriendRelationship]
-    ):
+    def __init__(self, state: ConnectionState, invitee: User | SteamID, relationship: FriendRelationship | None):
         self._state = state
         self.invitee = invitee
         self.relationship = relationship
@@ -69,28 +55,26 @@ class UserInvite(Invite):
 
     Attributes
     ----------
-    invitee: Union[:class:`~steam.User`, :class:`~steam.SteamID`]
+    invitee
         The user who sent the invite.
-    relationship: Optional[:class:`~steam.EFriendRelationship`]
+    relationship
         The relationship you have with the invitee. This ``None`` if the invite was sent while the bot is online.
     """
 
+    __slots__ = ()
+
     def __repr__(self) -> str:
         attrs = ("invitee",)
         resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
         return f"<UserInvite {' '.join(resolved)}>"
 
     async def accept(self) -> None:
-        """|coro|
-        Accepts the invite request.
-        """
+        """Accept the invite request."""
         await self._state.http.accept_user_invite(self.invitee.id64)
         self._state.client.user.friends.append(self.invitee)
 
     async def decline(self) -> None:
-        """|coro|
-        Declines the invite request.
-        """
+        """Decline the invite request."""
         await self._state.http.decline_user_invite(self.invitee.id64)
 
 
@@ -99,22 +83,25 @@ class ClanInvite(Invite):
 
     Attributes
     ----------
-    clan: Union[:class:`~steam.Clan`, :class:`~steam.SteamID`]
+    clan
         The clan to join.
-    invitee: Union[:class:`~steam.User`, :class:`~steam.SteamID`]
+    invitee
         The user who sent the invite.
-    relationship: :class:`~steam.EFriendRelationship`
+    relationship
         The relationship you have with the clan.
     """
 
+    # <a class="linkTitle" href="https://steamcommunity.com/games/581660">Super Meat Boy Forever</a>
+    # if game invite url is https://steamcommunity.com/games/581660
+
     __slots__ = ("clan",)
 
     def __init__(
         self,
         state: ConnectionState,
-        invitee: Union[User, SteamID],
-        clan: Union[Clan, SteamID],
-        relationship: Optional[EFriendRelationship],
+        invitee: User | SteamID,
+        clan: Clan | SteamID,
+        relationship: FriendRelationship | None,
     ):
         super().__init__(state, invitee, relationship)
         self.clan = clan
@@ -125,13 +112,9 @@ class ClanInvite(Invite):
         return f"<ClanInvite {' '.join(resolved)}>"
 
     async def accept(self) -> None:
-        """|coro|
-        Accepts the invite request.
-        """
+        """Accept the invite request."""
         await self._state.http.accept_clan_invite(self.clan.id64)
 
     async def decline(self) -> None:
-        """|coro|
-        Declines the invite request.
-        """
+        """Decline the invite request."""
         await self._state.http.decline_clan_invite(self.clan.id64)
