@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 The MIT License (MIT)
 
@@ -26,18 +24,23 @@ SOFTWARE.
 
 from __future__ import annotations
 
-from datetime import datetime
-from typing import TYPE_CHECKING, Any, Generator, Optional, Union
+import asyncio
+from collections.abc import Iterator, Sequence
+from datetime import datetime, timedelta
+from typing import TYPE_CHECKING, Any, Generic, TypeVar
 
-from typing_extensions import TypedDict
+from typing_extensions import TypeAlias, TypedDict
 
-from .enums import ETradeOfferState
+from .enums import TradeOfferState
 from .errors import ClientException, ConfirmationError
-from .game import Game
+from .game import Game, StatefulGame
 
 if TYPE_CHECKING:
+    from typing_extensions import Required
+
+    from .abc import BaseUser, SteamID
     from .state import ConnectionState
-    from .user import BaseUser, User
+    from .user import User
 
 
 __all__ = (
@@ -47,10 +50,10 @@ __all__ = (
     "TradeOffer",
 )
 
-Items = Union["Item", "Asset"]
+Items: TypeAlias = "Item | Asset"
+I = TypeVar("I", bound="Item")
 
 
-# TypedDicts to help visualise the data you receive
 class AssetToDict(TypedDict):
     assetid: str
     amount: int
@@ -65,8 +68,8 @@ class AssetDict(AssetToDict):
 
 
 class DescriptionDict(TypedDict, total=False):
-    instanceid: str
-    classid: str
+    instanceid: Required[str]
+    classid: Required[str]
     market_name: str
     currency: int
     name: str
@@ -83,6 +86,7 @@ class DescriptionDict(TypedDict, total=False):
     tradable: bool  # 1 vs 0
     marketable: bool  # same as above
     commodity: int  # might be a bool
+    fraudwarnings: list[str]
 
 
 class ItemDict(AssetDict, DescriptionDict):
@@ -93,7 +97,7 @@ class InventoryDict(TypedDict):
     assets: list[AssetDict]
     descriptions: list[DescriptionDict]
     total_inventory_count: int
-    success: int  # EResult
+    success: int  # Result
     rwgrsn: int  # p. much always -2
 
 
@@ -102,7 +106,7 @@ class TradeOfferDict(TypedDict):
     tradeid: str  # no clue what this is (its not the useful one)
     accountid_other: int
     message: str
-    trade_offer_state: int  # ETradeOfferState
+    trade_offer_state: int  # TradeOfferState
     expiration_time: int  # unix timestamps
     time_created: int
     time_updated: int
@@ -111,7 +115,7 @@ class TradeOfferDict(TypedDict):
     items_to_receive: list[ItemDict]
     is_our_offer: bool
     from_real_time_trade: bool
-    confirmation_method: int  # 2 is mobile not a clue what other values are
+    confirmation_method: int  # 2 is mobile 1 might be email? not a clue what other values are
 
 
 class Asset:
@@ -124,25 +128,22 @@ class Asset:
 
             Checks if two assets are equal.
 
-        .. describe:: x != y
-
-            Checks if two assets are not equal.
-
     Attributes
     -------------
-    game: :class:`~steam.Game`
+    game
         The game the item is from.
-    asset_id: :class:`str`
+    asset_id
         The assetid of the item.
-    amount: :class:`int`
+    amount
         The amount of the same asset there are in the inventory.
-    instance_id: :class:`str`
+    instance_id
         The instanceid of the item.
-    class_id: :class:`str`
+    class_id
         The classid of the item.
     """
 
-    __slots__ = ("game", "amount", "class_id", "asset_id", "instance_id", "name")
+    __slots__ = ("game", "amount", "class_id", "asset_id", "instance_id")
+    REPR_ATTRS = __slots__
 
     def __init__(self, data: AssetDict):
         self.asset_id = int(data["assetid"])
@@ -150,18 +151,11 @@ class Asset:
         self.amount = int(data["amount"])
         self.instance_id = int(data["instanceid"])
         self.class_id = int(data["classid"])
-        self.name = None  # so we don't raise AttributeError
 
     def __repr__(self) -> str:
-        attrs = (
-            "game",
-            "amount",
-            "class_id",
-            "asset_id",
-            "instance_id",
-        )
-        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
-        return f"<Asset {' '.join(resolved)}>"
+        cls = self.__class__
+        resolved = [f"{attr}={getattr(self, attr, None)!r}" for attr in cls.REPR_ATTRS]
+        return f"<{cls.__name__} {' '.join(resolved)}>"
 
     def __eq__(self, other: Any) -> bool:
         return isinstance(other, Asset) and self.instance_id == other.instance_id and self.class_id == other.class_id
@@ -184,26 +178,21 @@ class Item(Asset):
 
             Checks if two items are equal.
 
-        .. describe:: x != y
-
-            Checks if two items are not equal.
-
     Attributes
     -------------
-    name: Optional[:class:`str`]
+    name
         The market_name of the item.
-    display_name: Optional[:class:`str`]
-        The displayed name of the item. This could be different to
-        :attr:`Item.name` if the item is user re-nameable.
-    colour: Optional[:class:`int`]
+    display_name
+        The displayed name of the item. This could be different to :attr:`Item.name` if the item is user re-nameable.
+    colour
         The colour of the item.
-    descriptions: Optional[:class:`str`]
+    descriptions
         The descriptions of the item.
-    type: Optional[:class:`str`]
+    type
         The type of the item.
-    tags: Optional[:class:`str`]
+    tags
         The tags of the item.
-    icon_url: Optional[:class:`str`]
+    icon_url
         The icon_url of the item. Uses the large (184x184 px) image url.
     """
 
@@ -215,21 +204,16 @@ class Item(Asset):
         "icon_url",
         "display_name",
         "descriptions",
+        "fraud_warnings",
         "_is_tradable",
         "_is_marketable",
     )
+    REPR_ATTRS = ("name", *Asset.REPR_ATTRS)
 
     def __init__(self, data: ItemDict):
         super().__init__(data)
         self._from_data(data)
 
-    def __repr__(self) -> str:
-        asset_repr = super().__repr__()[7:-1]
-        attrs = ("name",)
-        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
-        resolved.append(asset_repr)
-        return f"<Item {' '.join(resolved)}>"
-
     def _from_data(self, data: ItemDict) -> None:
         self.name = data.get("market_name")
         self.display_name = data.get("name")
@@ -242,19 +226,103 @@ class Item(Asset):
             if "icon_url_large" in data
             else None
         )
+        self.fraud_warnings = data.get("fraudwarnings", [])
         self._is_tradable = bool(data.get("tradable", False))
         self._is_marketable = bool(data.get("marketable", False))
 
     def is_tradable(self) -> bool:
-        """:class:`bool`: Whether the item is tradable."""
+        """Whether the item is tradable."""
         return self._is_tradable
 
     def is_marketable(self) -> bool:
-        """:class:`bool`: Whether the item is marketable."""
+        """Whether the item is marketable."""
         return self._is_marketable
 
 
-class Inventory:
+class BaseInventory(Generic[I]):
+    """Base for all inventories."""
+
+    __slots__ = ("game", "items", "owner", "_state")
+
+    def __init__(self, state: ConnectionState, data: InventoryDict, owner: BaseUser, game: Game):
+        self._state = state
+        self.owner = owner
+        self.game = StatefulGame(state, id=game.id, name=game.name)
+        self._update(data)
+
+    def __repr__(self) -> str:
+        attrs = ("owner", "game")
+        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
+        return f"<{self.__class__.__name__} {' '.join(resolved)}>"
+
+    def __len__(self) -> int:
+        return len(self.items)
+
+    def __iter__(self) -> Iterator[I]:
+        return iter(self.items)
+
+    def __contains__(self, item: Asset) -> bool:
+        if not isinstance(item, Asset):
+            raise TypeError(
+                f"unsupported operand type(s) for 'in': {item.__class__.__qualname__!r} and {self.__class__.__name__!r}"
+            )
+        return item in self.items
+
+    def _update(self, data: InventoryDict) -> None:
+        self.items: Sequence[I] = []
+        for asset in data.get("assets", ()):
+            for item in data["descriptions"]:
+                if item["instanceid"] == asset["instanceid"] and item["classid"] == asset["classid"]:
+                    item.update(asset)
+                    self.items.append(Item(data=item))  # type: ignore
+                    break
+            else:
+                self.items.append(Asset(data=asset))  # type: ignore
+
+    async def update(self) -> None:
+        """Re-fetches the inventory."""
+        if not self.game:
+            return
+        data = await self._state.http.get_user_inventory(self.owner.id64, self.game.id, self.game.context_id)
+        self._update(data)
+
+    def filter_items(self, *names: str, limit: int | None = None) -> list[I]:
+        """A helper function that filters items by name from the inventory.
+
+        Parameters
+        ------------
+        names
+            The names of the items to filter for.
+        limit
+            The maximum amount of items to return. Checks from the front of the items.
+
+        Raises
+        -------
+        :exc:`ValueError`
+            You passed a limit and multiple item names.
+
+        Returns
+        ---------
+        The matching items.
+        """
+        if len(names) > 1 and limit:
+            raise ValueError("Cannot pass a limit with multiple items")
+        items = [item for item in self if item.name in names]
+        return items if limit is None else items[:limit]
+
+    def get_item(self, name: str) -> I | None:
+        """A helper function that gets an item or ``None`` if no matching item is found by name from the inventory.
+
+        Parameters
+        ----------
+        name
+            The item to get from the inventory.
+        """
+        item = self.filter_items(name, limit=1)
+        return item[0] if item else None
+
+
+class Inventory(BaseInventory[Item]):
     """Represents a User's inventory.
 
     .. container:: operations
@@ -269,114 +337,20 @@ class Inventory:
 
         .. describe:: y in x
 
-            Determines if an item is in the inventory based off of its class_id and instance_id.
+            Determines if an item is in the inventory based off of its :attr:`class_id` and :attr:`instance_id`.
 
 
     Attributes
     -------------
-    items: list[Union[:class:`Item`, :class:`Asset`]]
+    items
         A list of the inventory's items.
-    owner: :class:`~steam.User`
+    owner
         The owner of the inventory.
-    game: Optional[:class:`steam.Game`]
+    game
         The game the inventory the game belongs to.
     """
 
-    __slots__ = ("game", "items", "owner", "_state", "_total_inventory_count")
-
-    def __init__(self, state: ConnectionState, data: InventoryDict, owner: BaseUser):
-        self._state = state
-        self.owner = owner
-        self.items: list[Items] = []
-        self.game: Optional[Game]
-        self._update(data)
-
-    def __repr__(self) -> str:
-        attrs = ("owner", "game")
-        resolved = [f"{attr}={getattr(self, attr)!r}" for attr in attrs]
-        return f"<Inventory {' '.join(resolved)}>"
-
-    def __len__(self) -> int:
-        return self._total_inventory_count
-
-    def __iter__(self) -> Generator[Items, None, None]:
-        return (item for item in self.items)
-
-    def __contains__(self, item: Asset) -> bool:
-        if not isinstance(item, Asset):
-            raise TypeError("unsupported operand type(s) for 'in':" f" '{item.__class__.__qualname__}' and 'Inventory'")
-        return item in self.items
-
-    def _update(self, data: InventoryDict) -> None:
-        try:
-            self.game = Game(id=int(data["assets"][0]["appid"]))
-        except KeyError:  # they don't have an inventory in this game
-            self.game = None
-            self.items = []
-            self._total_inventory_count = 0
-        else:
-            for asset in data["assets"]:
-                found = False
-                for item in data["descriptions"]:
-                    if item["instanceid"] == asset["instanceid"] and item["classid"] == asset["classid"]:
-                        item.update(asset)
-                        self.items.append(Item(data=item))
-                        found = True
-                if not found:
-                    self.items.append(Asset(data=asset))
-            self._total_inventory_count = data["total_inventory_count"]
-
-    async def update(self) -> None:
-        """|coro|
-        Re-fetches the inventory.
-        """
-        data = await self._state.http.get_user_inventory(self.owner.id64, self.game.id, self.game.context_id)
-        self._update(data)
-
-    def filter_items(self, *names: str, **kwargs: Any) -> list[Item]:
-        """A helper function that filters and removes items by name from the inventory.
-
-        Parameters
-        ------------
-        *names: :class:`str`
-            The names of the items to filter for.
-        limit: Optional[:class:`int`]
-            The maximum amount of items to return. Checks from the front of the items.
-
-        Raises
-        -------
-        :exc:`ValueError`
-            You passed a limit and multiple item names.
-
-        Returns
-        ---------
-        list[:class:`Item`]
-            The removed matching items.
-        """
-        limit: int = kwargs.get("limit")
-        if limit and names:
-            raise ValueError("Cannot pass a limit with multiple items")
-        items = [item for item in self if item.name in names]
-        items = items if limit is None else items[:limit]
-        for item in items:
-            self.items.remove(item)
-        return items
-
-    def get_item(self, name: str) -> Optional[Item]:
-        """A helper function that gets and removes an item by name from the inventory.
-
-        Parameters
-        ----------
-        name: :class:`str`
-            The item to get from the inventory.
-
-        Returns
-        -------
-        Optional[:class:`Item`]
-            Returns the first item found with a matching name. Could be ``None`` if no matching item is found.
-        """
-        item = self.filter_items(name, limit=1)
-        return item[0] if item else None
+    __slots__ = ()
 
 
 class TradeOffer:
@@ -385,37 +359,45 @@ class TradeOffer:
 
     Parameters
     ----------
-    item_to_send: Optional[:class:`steam.Item`]
+    item_to_send
         The item to send with the trade offer.
-    item_to_receive: Optional[:class:`steam.Item`]
+    item_to_receive
         The item to receive with the trade offer.
-    items_to_send: Optional[list[:class:`steam.Item`]]
+    items_to_send
         The items you are sending to the other user.
-    items_to_receive: Optional[list[:class:`steam.Item`]]
-        The items you are sending to the other user.
-    token: Optional[:class:`str`]
+    items_to_receive
+        The items you are receiving from the other user.
+    token
         The the trade token used to send trades to users who aren't on the ClientUser's friend's list.
-    message: Optional[:class:`str`]
+    message
          The offer message to send with the trade.
 
     Attributes
-    -------------
-    partner: Union[:class:`~steam.User`, :class:`~steam.SteamID`]
+    ----------
+    partner
         The trade offer partner. This should only ever be a :class:`~steam.SteamID` if the partner's profile is private.
-    items_to_send: Union[list[:class:`Item`]]
+    items_to_send
         A list of items to send to the partner.
-    items_to_receive: Union[list[:class:`Item`]]
+    items_to_receive
         A list of items to receive from the partner.
-    state: :class:`~steam.ETradeOfferState`
-        The offer state of the trade for the possible types see :class:`~steam.ETradeOfferState`.
-    message: :class:`str`
+    state
+        The offer state of the trade for the possible types see :class:`~steam.TradeOfferState`.
+    message
         The message included with the trade offer.
-    id: :class:`int`
+    id
         The trade's offer ID.
-    expires: :class:`datetime.datetime`
+    created_at
+        The time at which the trade was created.
+    updated_at
+        The time at which the trade was last updated.
+    expires
         The time at which the trade automatically expires.
-    escrow: Optional[:class:`datetime.datetime`]
+    escrow
         The time at which the escrow will end. Can be ``None`` if there is no escrow on the trade.
+
+        Warning
+        -------
+        This isn't likely to be accurate, use :meth:`User.escrow` instead if possible.
     """
 
     __slots__ = (
@@ -426,6 +408,8 @@ class TradeOffer:
         "message",
         "token",
         "expires",
+        "updated_at",
+        "created_at",
         "items_to_send",
         "items_to_receive",
         "_has_been_sent",
@@ -436,38 +420,47 @@ class TradeOffer:
     def __init__(
         self,
         *,
-        message: Optional[str] = None,
-        token: Optional[str] = None,
-        item_to_send: Optional[Items] = None,
-        item_to_receive: Optional[Items] = None,
-        items_to_send: Optional[list[Items]] = None,
-        items_to_receive: Optional[list[Items]] = None,
+        message: str | None = None,
+        token: str | None = None,
+        item_to_send: Items | None = None,
+        item_to_receive: Items | None = None,
+        items_to_send: list[Items] | None = None,
+        items_to_receive: list[Items] | None = None,
     ):
-        self.items_to_receive: list[Items] = items_to_receive if items_to_receive else []
-        self.items_to_send: list[Items] = items_to_send if items_to_send else []
+        self.items_to_receive: list[Items] = items_to_receive or []
+        self.items_to_send: list[Items] = items_to_send or []
         if item_to_receive:
             self.items_to_receive.append(item_to_receive)
         if item_to_send:
             self.items_to_send.append(item_to_send)
-        self.message: str = message if message is not None else ""
-        self.token: Optional[str] = token
+        self.message: str | None = message or None
+        self.token: str | None = token
         self._has_been_sent = False
-        self.partner: Optional[User] = None
-        self.state = ETradeOfferState.Invalid
+        self.partner: User | SteamID | None = None
+        self.updated_at: datetime | None = None
+        self.created_at: datetime | None = None
+        self.escrow: timedelta | None = None
+        self.state = TradeOfferState.Invalid
 
     @classmethod
-    async def _from_api(cls, state: ConnectionState, data: TradeOfferDict) -> TradeOffer:
-        from .abc import SteamID
-
+    def _from_api(cls, state: ConnectionState, data: TradeOfferDict) -> TradeOffer:
         trade = cls()
         trade._has_been_sent = True
         trade._state = state
         trade._update(data)
-        trade.partner = await state.client.fetch_user(data["accountid_other"]) or SteamID(
-            data["accountid_other"]
-        )  # the account is private :(
+        trade.partner = int(data["accountid_other"])  # type: ignore
         return trade
 
+    def _update_from_send(
+        self, state: ConnectionState, data: dict[str, Any], partner: User, active: bool = True
+    ) -> None:
+        self.id = int(data["tradeofferid"])
+        self._state = state
+        self.partner = partner
+        self.state = TradeOfferState.Active if active else TradeOfferState.ConfirmationNeed
+        self.created_at = datetime.utcnow()
+        self._is_our_offer = True
+
     def __repr__(self) -> str:
         attrs = ("id", "state", "partner")
         resolved = [f"{attr}={getattr(self, attr, None)!r}" for attr in attrs]
@@ -478,9 +471,13 @@ class TradeOffer:
         self.id = int(data["tradeofferid"])
         expires = data.get("expiration_time")
         escrow = data.get("escrow_end_date")
+        updated_at = data.get("time_updated")
+        created_at = data.get("time_created")
         self.expires = datetime.utcfromtimestamp(expires) if expires else None
-        self.escrow = datetime.utcfromtimestamp(escrow) if escrow else None
-        self.state = ETradeOfferState(data.get("trade_offer_state", 1))
+        self.escrow = datetime.utcfromtimestamp(escrow) - datetime.utcnow() if escrow else None
+        self.updated_at = datetime.utcfromtimestamp(updated_at) if updated_at else None
+        self.created_at = datetime.utcfromtimestamp(created_at) if created_at else None
+        self.state = TradeOfferState.try_value(data.get("trade_offer_state", 1))
         self.items_to_send = [Item(data=item) for item in data.get("items_to_give", [])]
         self.items_to_receive = [Item(data=item) for item in data.get("items_to_receive", [])]
         self._is_our_offer = data.get("is_our_offer", False)
@@ -489,8 +486,7 @@ class TradeOffer:
         return isinstance(other, TradeOffer) and self._has_been_sent and other._has_been_sent and self.id == other.id
 
     async def confirm(self) -> None:
-        """|coro|
-        Confirms the :class:`TradeOffer`.
+        """Confirms the trade offer.
         This rarely needs to be called as the client handles most of these.
 
         Raises
@@ -503,16 +499,16 @@ class TradeOffer:
         self._check_active()
         if self.is_gift():
             return  # no point trying to confirm it
-        if not await self._state.get_and_confirm_confirmation(self.id):
+        if not await self._state.fetch_and_confirm_confirmation(self.id):
             raise ConfirmationError("No matching confirmation could be found for this trade")
         del self._state._confirmations[self.id]
 
     async def accept(self) -> None:
-        """|coro|
-        Accepts the :class:`TradeOffer`.
+        """Accepts the trade offer.
 
-        .. note::
-            This also calls :meth:`confirm` (if necessary) so you don't have to.
+        Note
+        ----
+        This also calls :meth:`confirm` (if necessary) so you don't have to.
 
         Raises
         ------
@@ -521,11 +517,11 @@ class TradeOffer:
         :exc:`~steam.ConfirmationError`
             No matching confirmation could not be found.
         """
-        self._check_active()
-        if self.state == ETradeOfferState.Accepted:
+        if self.state == TradeOfferState.Accepted:
             raise ClientException("This trade has already been accepted")
         if self.is_our_offer():
             raise ClientException("You cannot accept an offer the ClientUser has made")
+        self._check_active()
         resp = await self._state.http.accept_user_trade(self.partner.id64, self.id)
         if resp.get("needs_mobile_confirmation", False):
             for tries in range(5):
@@ -533,46 +529,43 @@ class TradeOffer:
                     await self.confirm()
                 except ConfirmationError:
                     break
+                except ClientException:
+                    await asyncio.sleep(tries * 2)
 
     async def decline(self) -> None:
-        """|coro|
-        Declines the :class:`TradeOffer`.
+        """Declines the trade offer.
 
         Raises
         ------
         :exc:`~steam.ClientException`
             The trade is either not active, already declined or not from the ClientUser.
         """
-        self._check_active()
-        if self.state == ETradeOfferState.Declined:
+        if self.state == TradeOfferState.Declined:
             raise ClientException("This trade has already been declined")
         if self.is_our_offer():
             raise ClientException("You cannot decline an offer the ClientUser has made")
+        self._check_active()
         await self._state.http.decline_user_trade(self.id)
 
     async def cancel(self) -> None:
-        """|coro|
-        Cancels the :class:`TradeOffer`
+        """Cancels the trade offer.
 
         Raises
         ------
         :exc:`~steam.ClientException`
-            The trade is either not active, already cancelled or is from the ClientUser.
+            The trade is either not active or already cancelled.
         """
-        self._check_active()
-        if self.state == ETradeOfferState.Canceled:
+        if self.state == TradeOfferState.Canceled:
             raise ClientException("This trade has already been cancelled")
-        if not self.is_gift():
-            raise ClientException("Offer wasn't created by the ClientUser and therefore cannot be canceled")
+        self._check_active()
         await self._state.http.cancel_user_trade(self.id)
 
     async def counter(self, trade: TradeOffer) -> None:
-        """|coro|
-        Counters a trade offer from an :class:`User`.
+        """Counter a trade offer from an :class:`User`.
 
         Parameters
         -----------
-        trade: :class:`TradeOffer`
+        trade
             The trade offer to counter with.
 
         Raises
@@ -580,27 +573,26 @@ class TradeOffer:
         :exc:`~steam.ClientException`
             The trade from the ClientUser or it isn't active.
         """
-        if not self._has_been_sent:
-            raise ClientException("This trade isn't active")
+        self._check_active()
         if self.is_our_offer():
             raise ClientException("You cannot counter an offer the ClientUser has made")
 
         to_send = [item.to_dict() for item in trade.items_to_send]
         to_receive = [item.to_dict() for item in trade.items_to_receive]
         resp = await self._state.http.send_trade_offer(
-            self.partner, to_send, to_receive, trade.token, trade.message, trade_id=self.id
+            self.partner, to_send, to_receive, trade.token, trade.message or "", trade_id=self.id
         )
         if resp.get("needs_mobile_confirmation", False):
-            await self._state.get_and_confirm_confirmation(int(resp["tradeofferid"]))
+            await self._state.fetch_and_confirm_confirmation(int(resp["tradeofferid"]))
 
     def is_gift(self) -> bool:
-        """:class:`bool`: Helper method that checks if an offer is a gift to the :class:`~steam.ClientUser`"""
-        return self.items_to_receive and not self.items_to_send
+        """Helper method that checks if an offer is a gift to the :class:`~steam.ClientUser`"""
+        return bool(self.items_to_receive and not self.items_to_send)
 
     def is_our_offer(self) -> bool:
-        """:class:`bool`: Whether the offer was created by the :class:`~steam.ClientUser`."""
+        """Whether the offer was created by the :class:`~steam.ClientUser`."""
         return self._is_our_offer
 
     def _check_active(self) -> None:
-        if self.state not in (ETradeOfferState.Active, ETradeOfferState.ConfirmationNeed) or not self._has_been_sent:
+        if self.state not in (TradeOfferState.Active, TradeOfferState.ConfirmationNeed) or not self._has_been_sent:
             raise ClientException("This trade is not active")
